import os

from aider.tools.utils.base_tool import BaseTool
from aider.tools.utils.helpers import (
    ToolError,
    apply_change,
    generate_unified_diff_snippet,
    handle_tool_error,
    validate_file_for_edit,
)


class Tool(BaseTool):
    NORM_NAME = "extractlines"
    SCHEMA = {
        "type": "function",
        "function": {
            "name": "ExtractLines",
            "description": "Extract lines from a source file and append them to a target file.",
            "parameters": {
                "type": "object",
                "properties": {
                    "source_file_path": {"type": "string"},
                    "target_file_path": {"type": "string"},
                    "start_pattern": {"type": "string"},
                    "end_pattern": {"type": "string"},
                    "line_count": {"type": "integer"},
                    "near_context": {"type": "string"},
                    "occurrence": {"type": "integer", "default": 1},
                    "dry_run": {"type": "boolean", "default": False},
                },
                "required": ["source_file_path", "target_file_path", "start_pattern"],
            },
        },
    }

    @classmethod
    def execute(
        cls,
        coder,
        source_file_path,
        target_file_path,
        start_pattern,
        end_pattern=None,
        line_count=None,
        near_context=None,
        occurrence=1,
        dry_run=False,
    ):
        """
        Extract a range of lines from a source file and move them to a target file.

        Parameters:
        - coder: The Coder instance
        - source_file_path: Path to the file to extract lines from
        - target_file_path: Path to the file to append extracted lines to (will be created if needed)
        - start_pattern: Pattern marking the start of the block to extract
        - end_pattern: Optional pattern marking the end of the block
        - line_count: Optional number of lines to extract (alternative to end_pattern)
        - near_context: Optional text nearby to help locate the correct instance of the start_pattern
        - occurrence: Which occurrence of the start_pattern to use (1-based index, or -1 for last)
        - dry_run: If True, simulate the change without modifying files

        Returns a result message.
        """
        tool_name = "ExtractLines"
        try:
            # --- Validate Source File ---
            abs_source_path, rel_source_path, source_content = validate_file_for_edit(
                coder, source_file_path
            )

            # --- Validate Target File ---
            abs_target_path = coder.abs_root_path(target_file_path)
            rel_target_path = coder.get_rel_fname(abs_target_path)
            target_exists = os.path.isfile(abs_target_path)

            if target_exists:
                # If target exists, validate it for editing
                try:
                    _, _, target_content = validate_file_for_edit(coder, target_file_path)
                except ToolError as e:
                    coder.io.tool_error(f"Target file validation failed: {str(e)}")
                    return f"Error: {str(e)}"
            else:
                # Target doesn't exist, start with empty content
                target_content = ""

            # --- Find Extraction Range ---
            if end_pattern and line_count:
                coder.io.tool_error("Cannot specify both end_pattern and line_count")
                return "Error: Cannot specify both end_pattern and line_count"

            source_lines = source_content.splitlines()

            start_pattern_line_indices = []
            for i, line in enumerate(source_lines):
                if start_pattern in line:
                    if near_context:
                        context_window_start = max(0, i - 5)
                        context_window_end = min(len(source_lines), i + 6)
                        context_block = "\n".join(
                            source_lines[context_window_start:context_window_end]
                        )
                        if near_context in context_block:
                            start_pattern_line_indices.append(i)
                    else:
                        start_pattern_line_indices.append(i)

            if not start_pattern_line_indices:
                err_msg = f"Start pattern '{start_pattern}' not found"
                if near_context:
                    err_msg += f" near context '{near_context}'"
                err_msg += f" in source file '{source_file_path}'."
                coder.io.tool_error(err_msg)
                return f"Error: {err_msg}"

            num_occurrences = len(start_pattern_line_indices)
            try:
                occurrence = int(occurrence)
                if occurrence == -1:
                    target_idx = num_occurrences - 1
                elif occurrence > 0 and occurrence <= num_occurrences:
                    target_idx = occurrence - 1
                else:
                    err_msg = (
                        f"Occurrence number {occurrence} is out of range for start pattern"
                        f" '{start_pattern}'. Found {num_occurrences} occurrences"
                    )
                    if near_context:
                        err_msg += f" near '{near_context}'"
                    err_msg += f" in '{source_file_path}'."
                    coder.io.tool_error(err_msg)
                    return f"Error: {err_msg}"
            except ValueError:
                coder.io.tool_error(
                    f"Invalid occurrence value: '{occurrence}'. Must be an integer."
                )
                return f"Error: Invalid occurrence value '{occurrence}'"

            start_line = start_pattern_line_indices[target_idx]
            occurrence_str = f"occurrence {occurrence} of " if num_occurrences > 1 else ""

            end_line = -1
            if end_pattern:
                for i in range(start_line, len(source_lines)):
                    if end_pattern in source_lines[i]:
                        end_line = i
                        break
                if end_line == -1:
                    err_msg = (
                        f"End pattern '{end_pattern}' not found after {occurrence_str}start"
                        f" pattern '{start_pattern}' (line {start_line + 1}) in"
                        f" '{source_file_path}'."
                    )
                    coder.io.tool_error(err_msg)
                    return f"Error: {err_msg}"
            elif line_count:
                try:
                    line_count = int(line_count)
                    if line_count <= 0:
                        raise ValueError("Line count must be positive")
                    end_line = min(start_line + line_count - 1, len(source_lines) - 1)
                except ValueError:
                    coder.io.tool_error(
                        f"Invalid line_count value: '{line_count}'. Must be a positive integer."
                    )
                    return f"Error: Invalid line_count value '{line_count}'"
            else:
                end_line = start_line  # Extract just the start line if no end specified

            # --- Prepare Content Changes ---
            extracted_lines = source_lines[start_line : end_line + 1]
            new_source_lines = source_lines[:start_line] + source_lines[end_line + 1 :]
            new_source_content = "\n".join(new_source_lines)

            # Append extracted lines to target content, ensuring a newline if target wasn't empty
            extracted_block = "\n".join(extracted_lines)
            if target_content and not target_content.endswith("\n"):
                target_content += "\n"  # Add newline before appending if needed
            new_target_content = target_content + extracted_block

            # --- Generate Diffs ---
            source_diff_snippet = generate_unified_diff_snippet(
                source_content, new_source_content, rel_source_path
            )
            target_insertion_line = len(target_content.splitlines()) if target_content else 0
            target_diff_snippet = generate_unified_diff_snippet(
                target_content, new_target_content, rel_target_path
            )

            # --- Handle Dry Run ---
            if dry_run:
                num_extracted = end_line - start_line + 1
                target_action = "append to" if target_exists else "create"
                coder.io.tool_output(
                    f"Dry run: Would extract {num_extracted} lines (from {occurrence_str}start"
                    f" pattern '{start_pattern}') in {source_file_path} and {target_action}"
                    f" {target_file_path}"
                )
                # Provide more informative dry run response with diffs
                return (
                    f"Dry run: Would extract {num_extracted} lines from {rel_source_path} and"
                    f" {target_action} {rel_target_path}.\nSource Diff"
                    f" (Deletion):\n{source_diff_snippet}\nTarget Diff"
                    f" (Insertion):\n{target_diff_snippet}"
                )

            # --- Apply Changes (Not Dry Run) ---
            # Apply source change
            source_metadata = {
                "start_line": start_line + 1,
                "end_line": end_line + 1,
                "start_pattern": start_pattern,
                "end_pattern": end_pattern,
                "line_count": line_count,
                "near_context": near_context,
                "occurrence": occurrence,
                "extracted_content": extracted_block,
                "target_file": rel_target_path,
            }
            source_change_id = apply_change(
                coder,
                abs_source_path,
                rel_source_path,
                source_content,
                new_source_content,
                "extractlines_source",
                source_metadata,
            )

            # Apply target change
            target_metadata = {
                "insertion_line": target_insertion_line + 1,
                "inserted_content": extracted_block,
                "source_file": rel_source_path,
            }
            target_change_id = apply_change(
                coder,
                abs_target_path,
                rel_target_path,
                target_content,
                new_target_content,
                "extractlines_target",
                target_metadata,
            )

            # --- Update Context ---
            coder.files_edited_by_tools.add(rel_source_path)
            coder.files_edited_by_tools.add(rel_target_path)

            if not target_exists:
                # Add the newly created file to editable context
                coder.abs_fnames.add(abs_target_path)
                coder.io.tool_output(
                    f"✨ Created and added '{target_file_path}' to editable context."
                )

            # --- Return Result ---
            num_extracted = end_line - start_line + 1
            target_action = "appended to" if target_exists else "created"
            coder.io.tool_output(
                f"✅ Extracted {num_extracted} lines from {rel_source_path} (change_id:"
                f" {source_change_id}) and {target_action} {rel_target_path} (change_id:"
                f" {target_change_id})"
            )
            # Provide more informative success response with change IDs and diffs
            return (
                f"Successfully extracted {num_extracted} lines from {rel_source_path} and"
                f" {target_action} {rel_target_path}.\nSource Change ID:"
                f" {source_change_id}\nSource Diff (Deletion):\n{source_diff_snippet}\nTarget"
                f" Change ID: {target_change_id}\nTarget Diff"
                f" (Insertion):\n{target_diff_snippet}"
            )

        except ToolError as e:
            # Handle errors raised by utility functions or explicitly raised here
            return handle_tool_error(coder, tool_name, e, add_traceback=False)
        except Exception as e:
            # Handle unexpected errors
            return handle_tool_error(coder, tool_name, e)



from typing import List

from aider.commands.utils.base_command import BaseCommand
from aider.commands.utils.helpers import format_command_result
from aider.utils import expand_glob_patterns


class LintCommand(BaseCommand):
    NORM_NAME = "lint"
    DESCRIPTION = "Lint and fix in-chat files or all dirty files if none in chat"

    @classmethod
    async def execute(cls, io, coder, args, **kwargs):
        """Execute the lint command with given parameters."""
        fnames = None

        # Get files from CLI arguments if available
        system_args = kwargs.get("system_args")
        if system_args:
            cli_files = getattr(system_args, "files", []) or []
            cli_file_arg = getattr(system_args, "file", []) or []
            all_cli_files = cli_files + cli_file_arg
            if all_cli_files:
                fnames = expand_glob_patterns(all_cli_files)

        if not coder.repo:
            io.tool_error("No git repository found.")
            return format_command_result(io, "lint", "No git repository found")

        if not fnames:
            fnames = coder.get_inchat_relative_files()

        # If still no files, get all dirty files in the repo
        if not fnames:
            fnames = coder.repo.get_dirty_files()

        if not fnames:
            io.tool_warning("No dirty files to lint.")
            return format_command_result(io, "lint", "No dirty files to lint")

        fnames = [coder.abs_root_path(fname) for fname in fnames]

        lint_coder = None
        for fname in fnames:
            try:
                errors = coder.linter.lint(fname)
            except FileNotFoundError as err:
                io.tool_error(f"Unable to lint {fname}")
                io.tool_output(str(err))
                continue

            if not errors:
                continue

            io.tool_output(errors)
            if not await io.confirm_ask(f"Fix lint errors in {fname}?", default="y"):
                continue

            # Commit everything before we start fixing lint errors
            if coder.repo.is_dirty() and coder.dirty_commits:
                # Use the commit command from registry
                from aider.commands import CommandRegistry

                await CommandRegistry.execute("commit", io, coder, "")

            if not lint_coder:
                lint_coder = await coder.clone(
                    # Clear the chat history, fnames
                    cur_messages=[],
                    done_messages=[],
                    fnames=None,
                )

            lint_coder.add_rel_fname(fname)
            await lint_coder.run_one(errors, preproc=False)
            lint_coder.abs_fnames = set()

        if lint_coder and coder.repo.is_dirty() and coder.auto_commits:
            # Use the commit command from registry
            from aider.commands import CommandRegistry

            await CommandRegistry.execute("commit", io, coder, "")

        return format_command_result(io, "lint", "Linting completed")

    @classmethod
    def get_completions(cls, io, coder, args) -> List[str]:
        """Get completion options for lint command."""
        # For lint command, we could return file paths for completion
        # For now, return empty list
        return []

    @classmethod
    def get_help(cls) -> str:
        """Get help text for the lint command."""
        help_text = super().get_help()
        help_text += "\nUsage:\n"
        help_text += "  /lint              # Lint all in-chat files or dirty files\n"
        help_text += "  /lint <files>      # Lint specific files\n"
        help_text += (
            "\nThis command lints files using the configured linter and offers to fix any errors"
            " found.\n"
        )
        help_text += (
            "If no files are specified, it lints all files in the chat or all dirty files in the"
            " repository.\n"
        )
        help_text += "For each file with lint errors, you'll be asked if you want to fix them.\n"
        return help_text



import argparse

import aider.prompts.utils.system as prompts
from aider import models
from aider.dump import dump  # noqa: F401


class ChatSummary:
    def __init__(self, models=None, max_tokens=1024):
        if not models:
            raise ValueError("At least one model must be provided")
        self.models = models if isinstance(models, list) else [models]
        self.max_tokens = max_tokens
        self.token_count = self.models[0].token_count

    def check_max_tokens(self, messages, max_tokens=None):
        if max_tokens is None:
            max_tokens = self.max_tokens

        if not max_tokens:
            return False

        sized = self.tokenize(messages)
        total = sum(tokens for tokens, _msg in sized)
        return total > max_tokens

    def tokenize(self, messages):
        sized = []
        for msg in messages:
            tokens = self.token_count(msg)
            sized.append((tokens, msg))
        return sized

    def count_tokens(self, messages):
        sized = self.tokenize(messages)
        total = sum(tokens for tokens, _msg in sized)
        return total

    async def summarize(self, messages, depth=0):
        messages = await self.summarize_real(messages)
        if messages and messages[-1]["role"] != "assistant":
            messages.append(dict(role="assistant", content="Ok."))
        return messages

    async def summarize_real(self, messages, depth=0):
        if not self.models:
            raise ValueError("No models available for summarization")

        sized = self.tokenize(messages)
        total = sum(tokens for tokens, _msg in sized)

        if total <= self.max_tokens:
            if depth == 0:
                # All fit, no summarization needed
                return messages
            # This is a chunk that's small enough to summarize in one go
            return await self.summarize_all(messages)

        min_split = 4
        if len(messages) <= min_split or depth > 4:
            return await self.summarize_all(messages)

        tail_tokens = 0
        split_index = len(messages)
        half_max_tokens = self.max_tokens // 2

        # Iterate over the messages in reverse order
        for i in range(len(sized) - 1, -1, -1):
            tokens, _msg = sized[i]
            if tail_tokens + tokens < half_max_tokens:
                tail_tokens += tokens
                split_index = i
            else:
                break

        # If we couldn't find a split point from the end, it's because the
        # last message was too big. So just split off the last message and
        # summarize the rest. This prevents infinite recursion.
        if split_index == len(messages):
            split_index = len(messages) - 1

        # Ensure the head ends with an assistant message
        while messages[split_index - 1]["role"] != "assistant" and split_index > 1:
            split_index -= 1

        if split_index <= min_split:
            return await self.summarize_all(messages)

        # Split head and tail
        head = messages[:split_index]
        tail = messages[split_index:]

        summary = await self.summarize_real(head, depth + 1)

        # If the combined summary and tail still fits, return directly
        new_messages = summary + tail

        sized_new = self.tokenize(new_messages)
        total_new = sum(tokens for tokens, _msg in sized_new)

        if total_new < self.max_tokens:
            return new_messages

        # Otherwise recurse with increased depth
        return await self.summarize_real(new_messages, depth + 1)

    async def summarize_all(self, messages):
        content = ""
        for msg in messages:
            role = msg["role"].upper()
            if role not in ("USER", "ASSISTANT"):
                continue
            if not msg.get("content"):
                continue
            content += f"# {role}\n"
            content += msg["content"]
            if not content.endswith("\n"):
                content += "\n"

        summarize_messages = [
            dict(role="system", content=prompts.summarize),
            dict(role="user", content=content),
        ]

        for model in self.models:
            try:
                summary = await model.simple_send_with_retries(summarize_messages)
                if summary is not None:
                    summary = prompts.summary_prefix + summary
                    return [dict(role="user", content=summary)]
            except Exception as e:
                print(f"Summarization failed for model {model.name}: {str(e)}")

        err = "summarizer unexpectedly failed for all models"
        print(err)
        raise ValueError(err)

    async def summarize_all_as_text(self, messages, prompt, max_tokens=None):
        content = ""
        for msg in messages:
            role = msg["role"].upper()
            if role not in ("USER", "ASSISTANT"):
                continue
            if not msg.get("content"):
                continue
            content += f"# {role}\n"
            content += msg["content"]
            if not content.endswith("\n"):
                content += "\n"

        summarize_messages = [
            dict(role="system", content=prompt),
            dict(role="user", content=content),
        ]

        for model in self.models:
            try:
                summary = await model.simple_send_with_retries(
                    summarize_messages, max_tokens=max_tokens
                )
                if summary is not None:
                    return summary
            except Exception as e:
                print(f"Summarization failed for model {model.name}: {str(e)}")

        raise ValueError("summarizer unexpectedly failed for all models")


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument("filename", help="Markdown file to parse")
    args = parser.parse_args()

    model_names = ["gpt-3.5-turbo", "gpt-4"]  # Add more model names as needed
    model_list = [models.Model(name) for name in model_names]
    summarizer = ChatSummary(model_list)

    with open(args.filename, "r") as f:
        text = f.read()

    summary = summarizer.summarize_chat_history_markdown(text)
    dump(summary)


if __name__ == "__main__":
    main()



"""
Command system for Aider.

This package contains individual command implementations that follow the
BaseCommand pattern for modular, testable command execution.
"""

import sys
import traceback
from pathlib import Path

from .add import AddCommand
from .agent import AgentCommand
from .architect import ArchitectCommand
from .ask import AskCommand
from .clear import ClearCommand
from .code import CodeCommand
from .command_prefix import CommandPrefixCommand
from .commit import CommitCommand
from .context import ContextCommand
from .context_blocks import ContextBlocksCommand
from .context_management import ContextManagementCommand
from .copy import CopyCommand
from .copy_context import CopyContextCommand
from .diff import DiffCommand

# Import and register commands
from .drop import DropCommand
from .editor import EditCommand, EditorCommand
from .exit import ExitCommand
from .git import GitCommand
from .help import HelpCommand
from .history_search import HistorySearchCommand
from .lint import LintCommand
from .list_sessions import ListSessionsCommand
from .load import LoadCommand
from .load_session import LoadSessionCommand
from .load_skill import LoadSkillCommand
from .ls import LsCommand
from .map import MapCommand
from .map_refresh import MapRefreshCommand
from .model import ModelCommand
from .models import ModelsCommand
from .multiline_mode import MultilineModeCommand
from .paste import PasteCommand
from .quit import QuitCommand
from .read_only import ReadOnlyCommand
from .read_only_stub import ReadOnlyStubCommand
from .reasoning_effort import ReasoningEffortCommand
from .remove_skill import RemoveSkillCommand
from .report import ReportCommand
from .reset import ResetCommand
from .run import RunCommand
from .save import SaveCommand
from .save_session import SaveSessionCommand
from .settings import SettingsCommand
from .test import TestCommand
from .think_tokens import ThinkTokensCommand
from .tokens import TokensCommand
from .undo import UndoCommand
from .utils.base_command import BaseCommand
from .utils.helpers import (
    CommandError,
    expand_subdir,
    format_command_result,
    get_available_files,
    glob_filtered_to_repo,
    parse_quoted_filenames,
    quote_filename,
    validate_file_access,
)
from .utils.registry import CommandRegistry
from .voice import VoiceCommand
from .weak_model import WeakModelCommand
from .web import WebCommand

# Register commands
CommandRegistry.register(DropCommand)
CommandRegistry.register(ClearCommand)
CommandRegistry.register(LsCommand)
CommandRegistry.register(DiffCommand)
CommandRegistry.register(ResetCommand)
CommandRegistry.register(CopyCommand)
CommandRegistry.register(PasteCommand)
CommandRegistry.register(SettingsCommand)
CommandRegistry.register(ReportCommand)
CommandRegistry.register(TokensCommand)
CommandRegistry.register(UndoCommand)
CommandRegistry.register(GitCommand)
CommandRegistry.register(RunCommand)
CommandRegistry.register(HelpCommand)
CommandRegistry.register(CommitCommand)
CommandRegistry.register(ModelsCommand)
CommandRegistry.register(ExitCommand)
CommandRegistry.register(QuitCommand)
CommandRegistry.register(VoiceCommand)
CommandRegistry.register(MapCommand)
CommandRegistry.register(MapRefreshCommand)
CommandRegistry.register(MultilineModeCommand)
CommandRegistry.register(EditorCommand)
CommandRegistry.register(EditCommand)
CommandRegistry.register(HistorySearchCommand)
CommandRegistry.register(ThinkTokensCommand)
CommandRegistry.register(LoadCommand)
CommandRegistry.register(SaveCommand)
CommandRegistry.register(ReasoningEffortCommand)
CommandRegistry.register(SaveSessionCommand)
CommandRegistry.register(ListSessionsCommand)
CommandRegistry.register(LoadSessionCommand)
CommandRegistry.register(ReadOnlyCommand)
CommandRegistry.register(ReadOnlyStubCommand)
CommandRegistry.register(AddCommand)
CommandRegistry.register(ModelCommand)
CommandRegistry.register(WeakModelCommand)
CommandRegistry.register(WebCommand)
CommandRegistry.register(LintCommand)
CommandRegistry.register(TestCommand)
CommandRegistry.register(ContextManagementCommand)
CommandRegistry.register(ContextBlocksCommand)
CommandRegistry.register(AskCommand)
CommandRegistry.register(CodeCommand)
CommandRegistry.register(ArchitectCommand)
CommandRegistry.register(ContextCommand)
CommandRegistry.register(AgentCommand)
CommandRegistry.register(CopyContextCommand)
CommandRegistry.register(CommandPrefixCommand)
CommandRegistry.register(LoadSkillCommand)
CommandRegistry.register(RemoveSkillCommand)

# Import SwitchCoder and Commands directly from commands.py
# We need to handle the circular import carefully

# Add parent directory to path to import commands.py directly
parent_dir = str(Path(__file__).parent.parent)
if parent_dir not in sys.path:
    sys.path.insert(0, parent_dir)

# Import the commands module directly
try:
    import importlib.util

    spec = importlib.util.spec_from_file_location(
        "aider.commands_module", Path(__file__).parent.parent / "commands.py"
    )
    commands_module = importlib.util.module_from_spec(spec)
    sys.modules["aider.commands_module"] = commands_module
    spec.loader.exec_module(commands_module)

    # Get the classes from the module
    Commands = getattr(commands_module, "Commands", None)
    SwitchCoder = getattr(commands_module, "SwitchCoder", None)

    if Commands is None or SwitchCoder is None:
        raise ImportError("Commands or SwitchCoder not found in commands.py")

except Exception as e:
    # Print the error for debugging
    print(f"Error importing commands.py: {e}")
    traceback.print_exc()

    # Fallback: define simple placeholder classes
    class SwitchCoder(Exception):
        def __init__(self, placeholder=None, **kwargs):
            self.kwargs = kwargs
            self.placeholder = placeholder

    class Commands:
        """Placeholder for Commands class defined in original commands.py"""

        def __init__(self, *args, **kwargs):
            # Accept any arguments but do nothing
            pass


__all__ = [
    "BaseCommand",
    "CommandRegistry",
    "CommandError",
    "quote_filename",
    "parse_quoted_filenames",
    "glob_filtered_to_repo",
    "validate_file_access",
    "format_command_result",
    "get_available_files",
    "expand_subdir",
    "DropCommand",
    "ClearCommand",
    "LsCommand",
    "DiffCommand",
    "ResetCommand",
    "CopyCommand",
    "PasteCommand",
    "SettingsCommand",
    "ReportCommand",
    "TokensCommand",
    "UndoCommand",
    "GitCommand",
    "RunCommand",
    "HelpCommand",
    "CommitCommand",
    "ModelsCommand",
    "ExitCommand",
    "QuitCommand",
    "VoiceCommand",
    "MapCommand",
    "MapRefreshCommand",
    "MultilineModeCommand",
    "EditorCommand",
    "EditCommand",
    "HistorySearchCommand",
    "ThinkTokensCommand",
    "LoadCommand",
    "SaveCommand",
    "ReasoningEffortCommand",
    "SaveSessionCommand",
    "ListSessionsCommand",
    "LoadSessionCommand",
    "ReadOnlyCommand",
    "ReadOnlyStubCommand",
    "AddCommand",
    "ModelCommand",
    "WeakModelCommand",
    "WebCommand",
    "LintCommand",
    "TestCommand",
    "ContextManagementCommand",
    "ContextBlocksCommand",
    "AskCommand",
    "CodeCommand",
    "ArchitectCommand",
    "ContextCommand",
    "AgentCommand",
    "CopyContextCommand",
    "CommandPrefixCommand",
    "LoadSkillCommand",
    "RemoveSkillCommand",
    "SwitchCoder",
    "Commands",
]
